@@ -197,6 +197,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
     // flag indicating whether the manager is considered alive
     private volatile boolean isActive;
 
+    @Override
     public void start( BundleContext bundleContext )
     {
         // track the log service using a ServiceTracker
@@ -278,7 +279,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         ConfigurationAdminFactory caf = new ConfigurationAdminFactory( this );
         Hashtable props = new Hashtable();
         props.put( Constants.SERVICE_PID, "org.apache.felix.cm.ConfigurationAdmin" );
-        props.put( Constants.SERVICE_DESCRIPTION, "Configuration Admin Service Specification 1.2 Implementation" );
+        props.put( Constants.SERVICE_DESCRIPTION, "Configuration Admin Service Specification 1.5 Implementation" );
         props.put( Constants.SERVICE_VENDOR, "Apache Software Foundation" );
         configurationAdminRegistration = bundleContext.registerService( ConfigurationAdmin.class.getName(), caf, props );
 
@@ -293,6 +294,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
     }
 
 
+    @Override
     public void stop( BundleContext bundleContext )
     {
 
@@ -681,10 +683,16 @@ public class ConfigurationManager implements BundleActivator, BundleListener
                 }
 
                 // ensure the service.pid and returned a cached config if available
-                ConfigurationImpl cfg = getCachedConfiguration( pid );
-                if ( cfg == null )
-                {
-                    cfg = new ConfigurationImpl( this, pmList[i], config );
+                ConfigurationImpl cfg;
+                if (!(pmList[i].isNotCachablePersistenceManager())) {
+                    cfg = getCachedConfiguration( pid );
+                    if (cfg == null) {
+                        cfg = new ConfigurationImpl(this, pmList[i], config);
+                        // add the to configurations cache if it wasn't in the cache
+                        cacheConfiguration(cfg);
+                    }
+                } else {
+                    cfg = new ConfigurationImpl(this, pmList[i], config);
                 }
 
                 // FELIX-611: Ignore configuration objects without props
@@ -786,6 +794,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
 
     // ---------- BundleListener -----------------------------------------------
 
+    @Override
     public void bundleChanged( BundleEvent event )
     {
         if ( event.getType() == BundleEvent.UNINSTALLED && handleBundleEvents )
@@ -818,7 +827,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
             List pmList = new ArrayList();
             CachingPersistenceManagerProxy[] pm;
 
-            ServiceReference[] refs = persistenceManagerTracker.getServiceReferences();
+            ServiceReference<?>[] refs = persistenceManagerTracker.getServiceReferences();
             if ( refs == null || refs.length == 0 )
             {
                 pm = new CachingPersistenceManagerProxy[0];
@@ -837,7 +846,12 @@ public class ConfigurationManager implements BundleActivator, BundleListener
                     Object service = persistenceManagerTracker.getService( refs[i] );
                     if ( service != null )
                     {
-                        pmList.add( new CachingPersistenceManagerProxy( ( PersistenceManager ) service ) );
+                        CachingPersistenceManagerProxy proxy = getProxyForPersistenceManager(( PersistenceManager ) service );
+                        if ( proxy == null )
+                        {
+                            proxy = new CachingPersistenceManagerProxy( ( PersistenceManager ) service );
+                        }
+                        pmList.add( proxy );
                     }
                 }
 
@@ -851,6 +865,17 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         return persistenceManagers;
     }
 
+    private CachingPersistenceManagerProxy getProxyForPersistenceManager(final PersistenceManager pm)
+    {
+        if (persistenceManagers != null) {
+            for (final CachingPersistenceManagerProxy pmProxy : persistenceManagers) {
+                if (pmProxy.getDelegatee() == pm) {
+                    return pmProxy;
+                }
+            }
+        }
+        return null;
+    }
 
     private ServiceReference getServiceReference()
     {
@@ -1110,7 +1135,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
     public void callPlugins( final Dictionary props, final ServiceReference sr, final String configPid,
         final String factoryPid )
     {
-        ServiceReference[] plugins = null;
+        ServiceReference<?>[] plugins = null;
         try
         {
             final String targetPid = (factoryPid == null) ? configPid : factoryPid;
@@ -1137,13 +1162,22 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         // call the plugins in order
         for ( int i = 0; i < plugins.length; i++ )
         {
-            ServiceReference pluginRef = plugins[i];
+            ServiceReference<?> pluginRef = plugins[i];
             ConfigurationPlugin plugin = ( ConfigurationPlugin ) bundleContext.getService( pluginRef );
             if ( plugin != null )
             {
+                // if cmRanking is below 0 or above 1000, ignore modifications from the plugin
+                boolean ignore = false;
+                Object rankObj = pluginRef.getProperty( ConfigurationPlugin.CM_RANKING );
+                if ( rankObj instanceof Integer )
+                {
+                    final int ranking = ( ( Integer ) rankObj ).intValue();
+                    ignore = (ranking < 0 ) || (ranking > 1000);
+                }
+
                 try
                 {
-                    plugin.modifyConfiguration( sr, props );
+                    plugin.modifyConfiguration( sr, ignore ? CaseInsensitiveDictionary.unmodifiable(props) : props );
                 }
                 catch ( Throwable t )
                 {
@@ -1390,6 +1424,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             for ( String pid : this.pids )
@@ -1475,6 +1510,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             for ( String factoryPid : this.factoryPids )
@@ -1486,63 +1522,65 @@ public class ConfigurationManager implements BundleActivator, BundleListener
                     factories = getTargetedFactories( factoryPid, sr );
                     for ( Factory factory : factories )
                     {
-                        for ( Iterator pi = factory.getPIDs().iterator(); pi.hasNext(); )
-                        {
-                            final String pid = ( String ) pi.next();
-                            ConfigurationImpl cfg;
-                            try
+                        synchronized (factory) {
+                            for ( Iterator pi = factory.getPIDs().iterator(); pi.hasNext(); )
                             {
-                                cfg = getConfiguration( pid );
-                            }
-                            catch ( IOException ioe )
-                            {
-                                log( LogService.LOG_ERROR, "Error loading configuration for {0}", new Object[]
-                                    { pid, ioe } );
-                                continue;
-                            }
+                                final String pid = ( String ) pi.next();
+                                ConfigurationImpl cfg;
+                                try
+                                {
+                                    cfg = getConfiguration( pid );
+                                }
+                                catch ( IOException ioe )
+                                {
+                                    log( LogService.LOG_ERROR, "Error loading configuration for {0}", new Object[]
+                                        { pid, ioe } );
+                                    continue;
+                                }
 
-                            // sanity check on the configuration
-                            if ( cfg == null )
-                            {
-                                log( LogService.LOG_ERROR,
-                                    "Configuration {0} referred to by factory {1} does not exist", new Object[]
-                                        { pid, factoryPid } );
-                                factory.removePID( pid );
-                                factory.storeSilently();
-                                continue;
-                            }
-                            else if ( cfg.isNew() )
-                            {
-                                // Configuration has just been created but not yet updated
-                                // we currently just ignore it and have the update mechanism
-                                // provide the configuration to the ManagedServiceFactory
-                                // As of FELIX-612 (not storing new factory configurations)
-                                // this should not happen. We keep this for added stability
-                                // but raise the logging level to error.
-                                log( LogService.LOG_ERROR, "Ignoring new configuration pid={0}", new Object[]
-                                    { pid } );
-                                continue;
-                            }
+                                // sanity check on the configuration
+                                if ( cfg == null )
+                                {
+                                    log( LogService.LOG_ERROR,
+                                        "Configuration {0} referred to by factory {1} does not exist", new Object[]
+                                            { pid, factoryPid } );
+                                    factory.removePID( pid );
+                                    factory.storeSilently();
+                                    continue;
+                                }
+                                else if ( cfg.isNew() )
+                                {
+                                    // Configuration has just been created but not yet updated
+                                    // we currently just ignore it and have the update mechanism
+                                    // provide the configuration to the ManagedServiceFactory
+                                    // As of FELIX-612 (not storing new factory configurations)
+                                    // this should not happen. We keep this for added stability
+                                    // but raise the logging level to error.
+                                    log( LogService.LOG_ERROR, "Ignoring new configuration pid={0}", new Object[]
+                                        { pid } );
+                                    continue;
+                                }
 
-                            /*
-                             * this code would catch targeted factory PIDs;
-                             * since this is not expected any way, we can
-                             * leave this out
-                             */
-                            /*
-                            else if ( !factoryPid.equals( cfg.getFactoryPid() ) )
-                            {
-                                log( LogService.LOG_ERROR,
-                                    "Configuration {0} referred to by factory {1} seems to belong to factory {2}",
-                                    new Object[]
-                                        { pid, factoryPid, cfg.getFactoryPid() } );
-                                factory.removePID( pid );
-                                factory.storeSilently();
-                                continue;
-                            }
-                            */
+                                /*
+                                 * this code would catch targeted factory PIDs;
+                                 * since this is not expected any way, we can
+                                 * leave this out
+                                 */
+                                /*
+                                else if ( !factoryPid.equals( cfg.getFactoryPid() ) )
+                                {
+                                    log( LogService.LOG_ERROR,
+                                        "Configuration {0} referred to by factory {1} seems to belong to factory {2}",
+                                        new Object[]
+                                            { pid, factoryPid, cfg.getFactoryPid() } );
+                                    factory.removePID( pid );
+                                    factory.storeSilently();
+                                    continue;
+                                }
+                                */
 
-                            provide( factoryPid, cfg );
+                                provide( factoryPid, cfg );
+                            }
                         }
                     }
                 }
@@ -1715,6 +1753,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             log( LogService.LOG_DEBUG, "Updating configuration {0} to revision #{1}", new Object[]
@@ -1804,6 +1843,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             List<ServiceReference<?>> srList = this.getHelper().getServices( getTargetedServicePid() );
@@ -1847,8 +1887,10 @@ public class ConfigurationManager implements BundleActivator, BundleListener
                 try
                 {
                     Factory factory = getOrCreateFactory( factoryPid.toString() );
-                    factory.removePID( pid );
-                    factory.store();
+                    synchronized (factory) {
+                        factory.removePID( pid );
+                        factory.store();
+                    }
                 }
                 catch ( IOException ioe )
                 {
@@ -1877,6 +1919,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             List<ServiceReference<?>> srList = this.getHelper().getServices( getTargetedServicePid() );
@@ -2007,6 +2050,7 @@ public class ConfigurationManager implements BundleActivator, BundleListener
         }
 
 
+        @Override
         public void run()
         {
             for ( int i = 0; i < listeners.length; i++ )
